@@ -75,9 +75,10 @@ class MakeVocab(object):
         self.vocab.save_dict(path=vocab_path)
 
 
-    def add_vectors(self, in_path, overwrite=False, workers=8, niter=2, min_count=10, window=10, vsize=300):
-        data = SimpleIter(in_path)
-        w2v = Word2Vec(data, window=window, min_count=min_count, workers=workers, size=vsize, iter=niter)
+    def add_vectors(self, in_path=None, w2v=None, overwrite=False, workers=8, niter=2, min_count=10, window=10, vsize=300):
+        if w2v is None:
+            data = SimpleIter(in_path)
+            w2v = Word2Vec(data, window=window, min_count=min_count, workers=workers, size=vsize, iter=niter)
 
         for word in w2v.wv.vocab.keys():
             if word in self.vocab:
